;************************************************************************************************************************************
; Display.asm : Code file for Display/Timer code																					*
; TODO: Initialize interrupt SFR in main code, can't be done here as it might screw up other sections interrupt setups				*
;************************************************************************************************************************************
	
	#include "General.inc"		;Include project settings
	#include "FanBus.inc"		;Required for ISREnd label
	#include "ROMTable.inc"		

;************************************************************************************************************************************
; Equ's and Defines																													*
;************************************************************************************************************************************

#Define		cDisplayDP		7			;Bit 7 controls DP in display registers
#Define		cDisplayOff		0x1A		;Display value that turns the segment drivers off
#Define 	cDisplayFanBase	0x14		;Constant for displaying Fan1, Base value for other fans 	

;************************************************************************************************************************************
; Exports																															*
;************************************************************************************************************************************

	;Registers
	GLOBAL	TimerInt
	GLOBAL	TimerSeconds
	GLOBAL	DisplayValue
	GLOBAL	DisplayOnes
	GLOBAL	DisplayTens
	GLOBAL	DisplayHund	
	GLOBAL	DisplayFan

	;Functions
	GLOBAL	DisplayInit
	GLOBAL	DisplayISR
	GLOBAL	DisplayConvert
	GLOBAL	DisplaySetFanW
	GLOBAL	DisplaySetTimerSecondsW

;************************************************************************************************************************************
; Variables																															*
;************************************************************************************************************************************
Bank1			UDATA					;Uninitialized variables in Bank 1
iTimerInt		RES	1					;Incremented every Timer2 Interrupt, for internal use
TimerInt		RES 1					;Decremented every Timer 2 interrupt, stops at 0, for general use, user code is allowed to write to this timer
TimerSeconds	RES 1					;Decremented every Second, stops at 0, for general use, user code is allowed to write to this timer
DisplayValue	RES	1					;The binairy value to be displayed 
DisplayOnes		RES 1					;The 'Ones' value to be displayed - Bit 7 controls DP
DisplayTens		RES	1					;The 'Tens' value to be displayed - Bit 7 controls DP
DisplayHund		RES 1					;The 'Hundreds' value to be displayed - Bit 7 controls DP
DisplayFan		RES	1					;The 'Fan' value to be displayed - Bit 7 Controls DP
iDisplayCurr	RES 1					;Holds the number of the current active display 
iDisplayTmp		RES 1					;Scratch variable

;************************************************************************************************************************************
PROG1	CODE	;Code 																												*
;************************************************************************************************************************************
; DisplayInit: Sets up timer 2 to generate an interrupt every 4mS (assuming 20Mhz PIC Clock), Configures TRIS's and outputs for 	*
;			   display.																												*
;			   Entry -> Bank X																										*
;			   Exit  ->	Bank 0																										*
;************************************************************************************************************************************
DisplayInit
	
	CLI							;Disable interrupts

	BANKSEL	PORTC				;Bank 0
	CLRF	PORTC				;Make sure PORTC is cleared
	CLRF	PORTA				;Make sure PORTA is cleared

	BANKSEL	TRISC				;Bank 1
	MOVLW	b'11000000'			
	MOVWF	TRISC				;All PORTC pins outputs, except USART pins
	CLRF	TRISA				;All PORTA pins outputs

	CLRF	TimerSeconds
	CLRF	TimerInt
	CLRF	iTimerInt			;Make sure Timers are cleared
	MOVLW	b'00000011'		
	MOVWF	iDisplayCurr		;Set currently active display to '4'
	MOVLW	cDisplayOff
	MOVWF	DisplayOnes
	MOVWF	DisplayTens
	MOVWF	DisplayHund
	MOVWF	DisplayFan			;Make sure all display values are set to "Off" for starters

	BSF		PIE1, TMR2IE		;Unmask Timer 2 interrupt
	BSF		INTCON, PEIE		;And enable periphial interrupts

	MOVLW	125 - 1
	MOVWF	PR2					;Set PR2 register for timer2 to divide by 125

	BANKSEL	TMR2				;Bank 0
	CLRF	TMR2				;Make sure TMR 2 is cleared
	MOVLW	b'01001110'
	MOVWF	T2CON				;Prescaler 1:16, Postscaler 1:10, Timer2 on 
	
	STI							;Enable interrupts
	RETURN						
	
;************************************************************************************************************************************
; DisplayISR: Display interrupt service routine. To be called by the interrupt handler when TMR2IF is set. Updates the display		*
;			  and the Timer registers.																								*
;  			  Entry -> Bank X																										*
;			  Exit  -> Bank 1																										*		  	
;************************************************************************************************************************************
DisplayISR

	BANKSEL	iDisplayCurr						;Bank 1
	INCF	iDisplayCurr, F						;Point to next display to update
	BTFSC	iDisplayCurr, 2						;Display 5 ?
	CLRF	iDisplayCurr						;Yes? then reset

	BANKISEL DisplayOnes						;Indirect adressing Bank0/1 	
	MOVF	iDisplayCurr, W
	ADDLW	DisplayOnes
	MOVWF	FSR									;Point indirect adressing to DisplayOnes + Display to update : Value to display now in INDF
	MOVF	INDF, W								;Copy to W
	ANDLW	b'01111111'							;Strip off the DP bit
	ROMTableSetPtrOffsetW	iDisplayValueTable	;Set ROMTable pointer to "iDisplayValueTable + W"
	CALL	ROMTableGet							;And get the translated value	(Returns in bank 1)
	MOVWF	iDisplayTmp							;Store it ...

	BANKSEL	PORTA								;Bank 0
	CLRF	PORTA								;Clear PORTA - This clears the CC driver bits => All displays off
	MOVWF	PORTC								;Copy the translated value to PORTC<0:5>, PORTC<6:7> under USART control/inputs
	BTFSC	INDF, cDisplayDP					;Test the DP bit of the untranslated value
	BSF		PORTC, 2							;If set - Set the DP bit on PORTC	

	BANKSEL	iDisplayTmp							;Bank 1
	MOVLW	b'11000000'
	ANDWF	iDisplayTmp, F						;Strip off iDisplayTmp bits 0:5
	BCF		STATUS, C							;Clear carry for RLF instruction
	RLF		iDisplayTmp, F
	RLF		iDisplayTmp, F
	RLF		iDisplayTmp, F						;Rotate iDisplayTmp<6:7> to iDisplayTmp<0:1>

	MOVF	iDisplayCurr, W						;Copy number of display to update to W
	ROMTableSetPtrOffsetW	iDisplayCCTable		;Set ROMTable pointer to "iDisplayCCTable + W"
	CALL	ROMTableGet							;Get the CC driver bit (Returns in Bank 1)
	IORWF	iDisplayTmp, W						;Mix the CC driver bit and the 2 remaining digit bits into W

	BANKSEL	PORTA								;Bank 0
	MOVWF	PORTA								;And copy to PORTA - This turns the display on

	BCF		PIR1, TMR2IF						;And clear Timer 2 interrupt flag while we are in Bank 0

	BANKSEL	iTimerInt							;Bank 1
	INCF	iTimerInt, F						;Increment the interrupt counter

	MOVF	TimerInt, F
	BTFSS	STATUS, Z							;TimerInt already reached 0 ? 
	DECF	TimerInt, F							;No? then decrement it 

	MOVLW	d'250'
	XORWF	iTimerInt, W							
	BTFSS	STATUS, Z							;Interrupt counter reached 250 ? (250 * 4mS = 1 second)
	GOTO	ISREnd								;No? then we're done 
												;Yes? ... then decrease TimerSeconds, if it isn't already 0
	MOVF	TimerSeconds, F
	BTFSS	STATUS, Z							;Is it 0 ?
	DECF	TimerSeconds, F						;No? then decrement it

	CLRF	iTimerInt							;Reset the interrupt counter
	GOTO	ISREnd								;And end					

;************************************************************************************************************************************
; DisplayConvert: Converts the binairy value in "DisplayValue" into Ones, Tens and Hundreds for displaying							*
;				  Copied from the PicList; www.piclist.com																			*
;				  Entry -> Bank X																									*
;				  Exit  -> Bank 1																									*
;************************************************************************************************************************************
DisplayConvert 

	CLI								;Disable interrupts, prevents display update with data we are still calculating here

	BANKSEL	iDisplayTmp				;Bank 1
	CLRF	iDisplayTmp
	BTFSC	DisplayFan, cDisplayDP		
	BSF		iDisplayTmp, 3
	BTFSC	DisplayHund, cDisplayDP
	BSF		iDisplayTmp, 2
	BTFSC	DisplayTens, cDisplayDP
	BSF		iDisplayTmp, 1
	BTFSC	DisplayOnes, cDisplayDP
	BSF		iDisplayTmp, 0			;Save the DP bits

	CLRF   	DisplayHund 
    SWAPF   DisplayValue, W			;w  = A0*16+A1
    ADDWF   DisplayValue, W 		;w  = A0+A1  
    ANDLW   00001111b 				;w  = A0+A1 % 16
    SKPNDC 							;if A0+A1 > 16
    ADDLW  	0x16 					;  w  += 16
    SKPNDC 							;if w % 16 > 10
    ADDLW 	0x06 					;  w  += 6
    ADDLW   0x06 					;w  += 6
    SKPDC 							;if w < 10
    ADDLW  	-0x06 					;  w  -= 6

    BTFSC   DisplayValue,4 
    ADDLW   0x16 - 1 + 0x6 
    SKPDC 
    ADDLW  	-0x06 

    BTFSC   DisplayValue, 5 
    ADDLW  	0x30 

    BTFSC   DisplayValue, 6 
    ADDLW  	0x60 

    BTFSC   DisplayValue, 7 
    ADDLW  	0x20 

    ADDLW   0x60 
    RLF     DisplayHund, F 
    BTFSS   DisplayHund, 0 
    ADDLW  	-0x60 

    MOVWF   DisplayOnes 			;iDisplayOnes now contains the Tens and Ones
	ANDLW	b'11110000'				;Strip off the Ones
	MOVWF	DisplayTens				;Copy to Tens register
	SWAPF	DisplayTens, F			;And swap the nibbles
	MOVLW	b'00001111'
	ANDWF	DisplayOnes, F			;Now strip of the Tens in the Ones register

    BTFSC   DisplayValue,7 
    INCF   	DisplayHund, F 		
									;Ones, Tens and Hund now contain the right values, now strip off preceding zeroes
	MOVLW	cDisplayOff
	MOVF	DisplayHund, F		
	BTFSS	STATUS, Z				;DisplayHund == 0 ?
	GOTO	RestoreDP				;No ? then end
	MOVWF	DisplayHund				;Yes? then turn it off

	MOVF	DisplayTens, F
	BTFSS	STATUS, Z				;DisplayTens == 0?
	GOTO	RestoreDP				;No ? then end
	MOVWF	DisplayTens				;Yes? then turn it off

RestoreDP
	
	BTFSC	iDisplayTmp, 0
	BSF		DisplayOnes, cDisplayDP
	BTFSC	iDisplayTmp, 1
	BSF		DisplayTens, cDisplayDP
	BTFSC	iDisplayTmp, 2
	BSF		DisplayHund, cDisplayDP
	BTFSC	iDisplayTmp, 3
	BSF		DisplayFan, cDisplayDP	;Restore the DP bits
	
	STI								;Enable interrupts
	RETURN	

;************************************************************************************************************************************
; DisplaySetFan: Sets the fan display to the fan number in W																		*
;				 Entry -> Bank X																									*
;				 Exit  -> Bank 1																									*
;************************************************************************************************************************************
DisplaySetFanW

	BANKSEL	DisplayFan				;Bank 1
	ADDLW	cDisplayFanBase			;Add Base adress for Fan1 in iDisplayValueTable to W
	BTFSC	DisplayFan, cDisplayDP	;Is the DP bit set ?
	IORLW	b'10000000'				;Yes? then OR it back into our result
	MOVWF	DisplayFan				;And copy to DisplayFan 
	RETURN

;************************************************************************************************************************************
; DisplaySetTimerSecondsW: Sets TimerSeconds to the value in W and resets iTimerInt												 	*
;						   Entry -> Bank X																							*
;						   Exit  -> Bank 1																							*
;************************************************************************************************************************************
DisplaySetTimerSecondsW

	BANKSEL	iTimerInt				;Bank 1
	CLRF	iTimerInt				;Clear iTimerInt to start from 0, without clearing our result can possibly be off by almost a second
	MOVWF	TimerSeconds			;And copy the value in W to TimerSeconds
	RETURN 

;************************************************************************************************************************************
; iDisplayValueTable: ROM Lookup table for display values																			*
;************************************************************************************************************************************
iDisplayValueTable	

	RETLW	b'01111011'			;'0'
	RETLW	b'00110000'			;'1'
	RETLW	b'10101011'			;'2'
	RETLW 	b'10111001'			;'3'
	RETLW	b'11110000'			;'4'
	RETLW   b'11011001'			;'5'
	RETLW   b'11011011'			;'6'
	RETLW	b'00111000'			;'7'
	RETLW 	b'11111011'			;'8'
	RETLW	b'11111001'			;'9'
	RETLW	b'11111010'			;'A'
	RETLW	b'11010011'			;'b'
	RETLW	b'01001011'			;'C'
	RETLW	b'10110011'			;'d'
	RETLW	b'11001011'			;'E'
	RETLW 	b'11001010'			;'F'
	RETLW	b'01000011'			;'L'
	RETLW	b'11011001'			;'S'
	RETLW	b'01000010'			;'I'
	RETLW	b'10010010'			;'n'
	RETLW	b'01000000'			; Fan 1   (0x14) Fanbase
	RETLW	b'00000010'			; Fan 2
	RETLW	b'00001000'			; Fan 3
	RETLW	b'00000001'			; Fan 4
	RETLW	b'00100000'			; Fan 5
	RETLW 	b'00010000'			; Fan 6
	RETLW	b'00000000'			; All Off (0x1A)
						
;************************************************************************************************************************************
; iDisplayNumTable: ROM Lookup table for Common Cathode activation bit																*
;************************************************************************************************************************************
iDisplayCCTable

	RETLW	b'00100000'			;Display 1 bit
	RETLW	b'00010000'			;Display 2 bit
	RETLW	b'00001000' 		;Display 3 bit
	RETLW	b'00000100'			;Display 4 bit

;************************************************************************************************************************************
 
	END		
