;************************************************************************************************************************************
; Eeprom.asm : Code file for the internal Eeprom code																				*
; !!!!!!!!!! : Code assumes all RAM addresses to read/write too are in Bank 0 or 1 and that the Eeprom address to read/write too	*
;			   is between 0x00 and 0x3F - Code originally written for the FanBus project on PIC16F870 wich only has 64 bytes of 	*
;			   Eeprom and no RAM in bank 2/3.																						*
; !!!!!!!!!! : The highest Eeprom byte (0x3F) is used for Eeprom validation. Before any write, this byte it set to a non-valid value*
;			   and only after successfull verification is it restored to the valid value (0x5A). A failed verify when writing or 	*
;			   an incorrect validation byte when reading will set the fEepromError flag. This prevents the device from running		*
;			   with incorrectly written eeprom data, caused by an power outage while writing to eeprom, for example					*
;************************************************************************************************************************************
	
	#include "General.inc"		;Include project settings
	#include "FanBus.inc"		;Required for ISREnd label

;************************************************************************************************************************************
; Equ's and Defines																													*
;************************************************************************************************************************************

#Define	fEepromError			EepromFlags, 0		;Flag indicating an error occured during an Eeprom function
#Define fEepromWriteBusy		EepromFlags, 1		;Flag indicating an interrupt based write operation is currently in progress
#Define	ifEepromVerifynRead		EepromFlags, 7		;Flag switching between read (0) and verify (1) in the EepromRead function
#Define icEepromValidation		0x3F				;The address, in eeprom, of the eeprom vaidation byte
#Define icEepromValid			0x5A				;The value icEepromValidation byte should hold to indicate valid eeprom data
#Define icEepromInvalid			0xFF				;The value written to icEepromValidation to indicate the data is invalid 

;************************************************************************************************************************************
; Exports																															*
;************************************************************************************************************************************

	;Registers
	GLOBAL	EepromAddr
	GLOBAL	EepromRamAddr
	GLOBAL	EepromAmount
	GLOBAL	EepromFlags

	;Functions
	GLOBAL	EepromInit
	GLOBAL	EepromRead
	GLOBAL	EepromWrite
	GLOBAL	EepromISR

;************************************************************************************************************************************
; Variables																															*
;************************************************************************************************************************************
Bank1			UDATA					;Uninitialized variables in Bank 1
EepromAddr		RES 1					;The eeprom address to read/write too
EepromRamAddr	RES 1					;The RAM address to read/write too
EepromAmount	RES	1					;The amount of bytes to read/write
EepromFlags		RES 1					;Eeprom flags
iEepromTmp		RES 1					;Scratch variable 

;************************************************************************************************************************************
PROG1	CODE	;Code 																												*
;************************************************************************************************************************************
; EepromInit: Makes sure the ifEepromVerifynRead is set to Read on startup and the fEepromWriteBusy flag is cleared					*
;			  Entry -> Bank X																										*
;			  Exit  -> Bank 1																										*
;************************************************************************************************************************************
EepromInit

	BANKSEL	EepromFlags					;Bank 1
	CLRF	EepromFlags				
	RETURN

;************************************************************************************************************************************
; iEepromReadW: Reads the byte in Eeprom at address EEADR into W																	*
;				Entry -> Bank X																										*
;			    Exit  -> Bank 2																										*
;************************************************************************************************************************************
iEepromReadW

	BANKSEL	EEADRH						;Bank 2
	CLRF	EEADRH						;Make sure the high byte of the Eeprom address is cleared

	BANKSEL	EECON1						;Bank 3
	BCF		EECON1, EEPGD				;Set Eeprom operations to data eeprom, not program memory eeprom
	BSF		EECON1, RD					;Perform Eeprom read operation

	BANKSEL	EEDATA						;Bank 2
	MOVF	EEDATA, W					;Copy Eeprom byte to W
	RETURN	

;************************************************************************************************************************************
; iEepromWrite: Writes the byte in EEDATA to Eeprom at address EEADR																*
;				Entry -> Bank X																										*
;				Exit  -> Bank 3																										*
;************************************************************************************************************************************
iEepromWrite

	BANKSEL	EEADRH						;Bank 2
	CLRF	EEADRH						;Make sure the high byte of the Eeprom address is cleared

	BANKSEL	EECON1						;Bank 3
	BCF		EECON1, EEPGD				;Set Eeprom operations to data eeprom, not program memory eeprom
	BSF		EECON1, WREN				;Enable writes
	
	CLI									;Make sure interrupts are disabled

	MOVLW	0x55
	MOVWF	EECON2
	MOVLW	0xAA
	MOVWF	EECON2						;Required operation ! Refer to datasheet	
	BSF		EECON1, WR					;Perform Eeprom write operation

	;STI								;Do not re-enable interrupts as this will trigger an interrupt inside an interrupt when
										;doing interrupt based eeprom writing - Interrupts will be reenabled by the RETFIE instruction automatically
	RETURN
					
;************************************************************************************************************************************
; EepromRead: Reads EepromAmount of bytes from Eeprom at address EepromAddr into RAM at address EepromRamAddr when 					*
;			  ifEepromVerifynRead is clear. Verifies EepromAmount bytes between EepromAddr and EepromRadAddr when 				    *
;			  ifEepromVerifynRead is set. Sets fEepromError on failed verify, when Eeprom validation value is not correct or when	*
;			  a write operation is currently in progress																			*
;			  Entry -> Bank X																										*
;			  Exit  -> Bank 1 																										* 
;************************************************************************************************************************************
EepromRead

	BANKSEL	EEADR						;Bank 2
	MOVLW	icEepromValidation
	MOVWF	EEADR						;Load address of eeprom validation byte
	CALL	iEepromReadW				;And read the validation byte (Returns in Bank 2)

	XORLW	icEepromValid				;Xor the validation byte with the icEepromValid constant => Z flag set on match

	BANKSEL	EepromFlags					;Bank 1
	BSF		fEepromError				;Assume validation value is incorrect
	BTFSC	ifEepromVerifynRead			;In case we are verifying ...
	GOTO	iEepromValidForRead			;... skip the test and continue, Validation value WILL purposely be wrong when verifying
	BTFSS	STATUS, Z					;icEepromValidation == icEepromValid ? 
	GOTO	iEepromReadEnd				;No ? Then eeprom cant be trusted - Exit with error flag set

	BTFSC	fEepromWriteBusy			;Is a interrupt based write operation currently in progress ?
	GOTO	iEepromReadEnd				;Yes ? then exit with error flag set , caller must wait for the write to finish						

iEepromValidForRead

	BCF		fEepromError				;Clear any previous errors
	
	BCF		STATUS, IRP					;Set indirect addressing to bank 0 and 1
	MOVF	EepromRamAddr, W
	MOVWF	FSR							;Point indirect addressing to EepromRamAddr
	
	MOVF	EepromAddr, W
	BANKSEL	EEADR						;Bank 2
	MOVWF	EEADR						;Copy EepromAddr to EEADR

iEepromReadNext

	CALL	iEepromReadW				;Perform the read operation (Returns in Bank 2)
	INCF	EEADR, F					;Already point the Eeprom adress to the next byte for the next pass (if there is one)

	XORWF	INDF, F						;Xor the retrieved byte (in W) with the byte in RAM => Z flag set on match
	MOVWF	INDF 						;Copy Eeprom byte to RAM
	
	BANKSEL	EepromFlags					;Bank 1
	BSF		fEepromError				;Assume we are verifying and had a mismatch
	BTFSC	STATUS, Z					;Did we have a mismatch? 
	BCF		fEepromError				;No? Then change our assumption

	BTFSS	ifEepromVerifynRead			;Are we even verifying ? 
	BCF		fEepromError				;No? Then we can't have any errors

	BTFSC	fEepromError				;If we are verifying AND we had a mismatch ...
	GOTO	iEepromReadEnd				;...there's no sense in continueing

	INCF	FSR, F						;Increment RAM pointer for next pass (if there is one)

	DECFSZ	EepromAmount, F				;Are we done ?
	GOTO	iEepromReadNext				;No? then do another pass
	
iEepromReadEnd

	BCF		ifEepromVerifynRead			;Make sure the function is back into read-mode for the next call	
	RETURN								;Otherwise end

;************************************************************************************************************************************
; EepromWrite: Writes EepromAmount of bytes from RAM at address EepromRamAddr into eeprom at EepromAddr. Sets the Validation byte 	*
;			   to 'invalid' beforce commencing. This ensures that any interrupted/not complete writes results in an error report 	*
;			   when trying to read the corrupted data. 																				*
;			   Interrupt based eeprom writing - This part does the setup , the rest of the write is handled by EepromISR			*
;			   fEepromWriteBusy is set during the write operation - No other eeprom operations are to be performed during this time *
;			   When the function is called while another write is already in progress or when the fEepromError flag is set from		*
;			   a previous error, Eeprom interrupts are disabled (effectively halting all writes in progress) and the function 		*
;			   returns with fEepromError set and fWriteBusy clear - Fatal error!													*
;			   When the operation is complete fEepromWriteBusy is cleared and the written data is verified. fEepromError will be	*
;			   set if the verify failed, clear on success																			*	
;			   Entry -> Bank X																										*
;			   Exit  -> Bank 1																										*
;************************************************************************************************************************************
EepromWrite

	BANKSEL	EepromFlags					;Bank 1
	BTFSC	fEepromError				;A previous error occured ? 			
	GOTO	iEepromWriteDenied			;Yes? then deny this write - end with fatal error
	BTFSC	fEepromWriteBusy			;Another write is already in progress ? 									
	GOTO	iEepromWriteDenied			;Yes? then deny this write - end with fatal error
										;No? then perform the write
	BANKSEL	EEADR						;Bank 2
	MOVLW	icEepromValidation
	MOVWF	EEADR						;Load address of eeprom validation byte
	MOVLW	icEepromInvalid
	MOVWF	EEDATA						;And load icEepromInvalid as data
	CALL	iEepromWrite				;And perform the write - Eeprom data is marked invalid now (Returns in Bank 3)

	BANKSEL	EepromFlags					;Bank 1
	BSF		fEepromWriteBusy			;Flag indicating that a write is in progress

	MOVF	EepromAmount, W
	ADDLW	0x01
	MOVWF	iEepromTmp					;Copy EepromAmount + 1 to iEepromTmp (+1 because we DECFSZ first and read after in EepromISR)

	BANKSEL	PIE2						;Bank 1	
	BSF		PIE2, EEIE					;Enable eeprom interrupts
	BSF		INTCON, PEIE				;Enable peripheral interrupts
	STI									;Enable global interrupts
	RETURN

iEepromWriteDenied
										;Write denied because of fEepromError flag already set or starting a write when another write is already in progress...
	BSF		fEepromError				;Then set the EepromError flag...
	BCF		fEepromWriteBusy			;All writes in progress will be cancelled...clear write in progress flag

	BANKSEL	PIE2						;Bank 1
	BCF		PIE2, EEIE					;...And disable all further eeprom writes - Fatal error 
	RETURN								

;************************************************************************************************************************************
; EepromISR: Eeprom interrupt service routine. To be called by the interrupt handler when EEIF is set. 								*
;			 Writes the next byte to be written to eeprom. Verifies the written data, fEepromError is set on failure, 				*
;			 clear on success. Sets the validation byte to 'valid' after successfull verification. 									*
;			 Entry -> Bank X																										*
;			 Exit  -> Bank 0																										*
;************************************************************************************************************************************
EepromISR			

	BANKSEL	iEepromTmp					;Bank 1
	MOVF	iEepromTmp, F			
	BTFSS	STATUS, Z					;Does this interrupt come upon completion of the validation byte ?
	GOTO	iEepromISRTestForLast		;No? then it is a data byte interrupt - Continue to code that tests for the last byte
	
iEepromISREndFlagsClear
										;Yes? ...
	BCF		fEepromWriteBusy			;...then clear the write busy flag - New eeprom functions are allowed again
	BCF		PIE2, EEIE					;And disable further eeprom interrupts
	GOTO	iEepromISREnd				;Write completed

iEepromISRTestForLast

	DECFSZ	iEepromTmp, F				;Does this interrupt come upon completion of the last data byte ?
	GOTO	iEepromISRWrite				;No? Then write the next byte
										;Yes? Then verify
	BSF		ifEepromVerifynRead			;Set EepromRead to verify
	CALL	EepromRead					;Perform verify

	BTFSC	fEepromError				;Error on verify ? 
	GOTO	iEepromISREndFlagsClear		;Yes ? Then end with fEepromError flag set, but clear the fEepromWriteBusy flag and Eeprom interrupts
										;No? verify ok! Set the validation byte to valid
	BANKSEL	EEADR						;Bank 2
	MOVLW	icEepromValidation
	MOVWF	EEADR						;Load address of eeprom validation byte
	MOVLW	icEepromValid
	MOVWF	EEDATA						;And load icEepromValid as data
	CALL	iEepromWrite				;And perform the write - Eeprom data is marked valid now (Returns in Bank 3)
	
	GOTO	iEepromISREnd				;And exit 

iEepromISRWrite

	BCF		STATUS, IRP					;Set indirect addressing to bank 0 and 1
	MOVF	iEepromTmp, W
	SUBWF	EepromAmount, W
	ADDWF	EepromRamAddr, W			;W = EepromRamAddr + (EepromAmount - iEepromTmp) => Ram address of byte we want to save to eeprom
	MOVWF	FSR							;Load into indirect addressing
						
	MOVF	iEepromTmp, W
	SUBWF	EepromAmount, W
	ADDWF	EepromAddr, W
	BANKSEL	EEADR		
	MOVWF	EEADR						;EEADR = EepromAddr + (EepromAmount - iEepromTmp) => Eeprom address to save byte into				
	
	MOVF	INDF, W					
	MOVWF	EEDATA						;Copy the byte we have to save to EEDATA
	CALL	iEepromWrite				;And perform the write (Returns in Bank 3)

iEepromISREnd

	BANKSEL	PIR2						;Bank 0
	BCF		PIR2, EEIF					;Clear the eeprom interrupt flag
	GOTO	ISREnd						;And end
	
;************************************************************************************************************************************
	
	END		
