;************************************************************************************************************************************
; FanSpeed.asm : Code file for the fan speed management code																		*
;				 The code is designed to handle 3 preset slots																		*
;************************************************************************************************************************************
	
	#include "General.inc"		;Include project settings
	#include "ErrorTrap.inc"	;To trap fatal eeprom errors
	#include "Eeprom.inc"		;Settings are saved/loaded too/from eeprom

;************************************************************************************************************************************
; Equ's and Defines																													*
;************************************************************************************************************************************

#Define		icFanSpeedPresetEepromAddr		0x3E				;The eeprom address where the currently active preset number is stored

#Define 	ifFanSpeedChanged				iFanSpeedFlags, 0	;Flag indicating the currently loaded preset's values have been changed 

;************************************************************************************************************************************
; Exports																															*
;************************************************************************************************************************************

	;Registers
	GLOBAL	FanSpeedValues
	GLOBAL	FanSpeedPreset
	GLOBAL	FanSpeedPresetToLoad
	GLOBAL 	FanSpeedSelectedValue

	;Functions
	GLOBAL	FanSpeedInit
	GLOBAL	FanSpeedLoadPreset
	GLOBAL	FanSpeedSavePreset
	GLOBAL	FanSpeedErrorTrap
	GLOBAL	FanSpeedSelectLast
	GLOBAL	FanSpeedSelectNext
	GLOBAL	FanSpeedGetSelectedW
	GLOBAL	FanSpeedDecSelected
	GLOBAL	FanSpeedPresetSelectLast
	GLOBAL	FanSpeedPresetSelectNext

;************************************************************************************************************************************
; Variables																															*
;************************************************************************************************************************************
Bank1					UDATA			;Uninitialized variables in Bank 1
FanSpeedValues			RES 6			;The speed values for every of the 6 fans
FanSpeedPreset			RES 1			;Value holding the number of the currently active/loaded preset
FanSpeedPresetToLoad	RES 1			;Value holding the number of the preset to load in the FanSpeedLoadPreset function
FanSpeedSelectedValue	RES 1			;The currently selected fanspeed value for manipulation/retrieval
iFanSpeedFlags			RES 1			;FanSpeed flags

;************************************************************************************************************************************
PROG1	CODE	;Code 																												*
;************************************************************************************************************************************
; FanSpeedInit:	Fan speed initialization code. Loads the last active preset slot number from eeprom, then loads that actual slot	*
;				number's data from eeprom. Also calles the EepromInit code.															*
;				Entry -> Bank X																										*
;			    Exit  -> Bank 1																										*
;************************************************************************************************************************************
FanSpeedInit

	CALL	EepromInit					;Initialize the eeprom module (Returns in Bank 1)

	CLRF	FanSpeedPreset				;Make sure FanSpeedPreset = 0

	MOVLW	icFanSpeedPresetEepromAddr
	MOVWF	EepromAddr					;Eeprom address = icFanSpeedPresetEepromAddr

	MOVLW	FanSpeedPresetToLoad		
	MOVWF	EepromRamAddr 				;Eeprom operations RAM address = FanSpeedPresetToLoad

	MOVLW	0x01
	MOVWF	EepromAmount				;1 byte to load
	CALL	EepromRead					;Perform the read (Returns in Bank 1)
	CALL	FanSpeedErrorTrap			;Test for eeprom errors and trap them (Returns in Bank 1)
										;FanSpeedPresetToLoad now holds the slotnumber we want to load, so all we need is ...
	CALL	FanSpeedLoadPreset			;...A call to FanSpeedLoadPreset (Returns in Bank 1)
	RETURN	

;************************************************************************************************************************************
; FanSpeedLoadPreset: Loads the preset slot number FanSpeedPresetToLoad from eeprom into FanSpeedValues. Updates FanSpeedPreset		*
;					  to hold the new slot number. Writes the newly loaded and thus active slot number to eeprom on adress			*
;					  icFanSpeedPresetEepromAddr - Allowing the device to start up with the same preset as it was shut down with	*
;					  Only to be called when fFanSpeedBusy is clear ! Calling when another write is in progress will result			*
;					  in a fatal error trap!!																						*
;					  If the FanSpeedPresetToLoad == the currently loaded FanSpeedPreset then no action is taken.					*
;					  If the icFanSpeedPresetEepromAddr byte in eeprom == FanSpeedPresetToLoad (possible on FanSpeedInit call)		*
;					  then the icFanSpeedPresetEepromAddr is not saved to prevent eeprom wear										*
;					  Clears the ifFanSpeedChanged flag, The loaded data is unchanged yet											*
;					  Entry -> Bank X																								*
;					  Exit  -> Bank 1																								*
;************************************************************************************************************************************
FanSpeedLoadPreset

	BANKSEL	FanSpeedPresetToLoad		;Bank 1
	MOVF	FanSpeedPresetToLoad, W
	XORWF	FanSpeedPreset, W			
	BTFSC	STATUS, Z					;FanSpeedPresetToLoad == FanSpeedPreset ? 
	RETURN								;Yes? then nothing to do
										;No? then continue
	BCF		ifFanSpeedChanged			;Clear the Changed flag - The preset we are going to load from eeprom is upto date

	MOVF	FanSpeedPresetToLoad, W
	MOVWF	FanSpeedPreset				;FanSpeedPreset = FanSpeedPresetToLoad
	CALL	iFanSpeedCalcEepromAddr		;Calculate the required eeprom address, save to EepromAddr (Returns in Bank 1)

	MOVLW	FanSpeedValues
	MOVWF	EepromRamAddr				;Load the ram address of FanSpeedValues into EepromRamAddr

	MOVLW	0x06
	MOVWF	EepromAmount				;6 bytes to load 
	CALL	EepromRead					;Perform the read (Returns in Bank 1)
	CALL	FanSpeedErrorTrap			;Test for eeprom errors and trap them (Returns in Bank 1)

	MOVLW	icFanSpeedPresetEepromAddr
	MOVWF	EepromAddr					;EepromAddr = icFanSpeedPresetEepromAddr

	MOVLW	FanSpeedPreset
	MOVWF	EepromRamAddr				;EepromRamAddr = FanSpeedPreset (we use FanSpeedPreset as a scratch variable)
	
	MOVLW	0x01
	MOVWF	EepromAmount				;1 Byte to load
	CALL	EepromRead					;Perform the read (Returns in Bank 1)
	CALL	FanSpeedErrorTrap			;Test for eeprom errors and trap them (Returns in Bank 1)
										;FanSpeedPreset now holds the value in eeprom @ icFanSpeedPresetEepromAddr
	MOVF	FanSpeedPresetToLoad, W
	XORWF	FanSpeedPreset, W
	BTFSC	STATUS, Z					;FanSpeedPresetToLoad == FanSpeedPreset ?(icFanSpeedPresetEepromAddr)
	RETURN								;Yes? then we don't need to save the active slot nr to icFanSpeedPresetEepromAddr (prevents eeprom wear)
										;No? then save the active slot number to icFanSpeedPresetEepromAddr
	MOVLW	icFanSpeedPresetEepromAddr
	MOVWF	EepromAddr					;EepromAddr = icFanSpeedPresetEepromAddr

	MOVLW	FanSpeedPresetToLoad
	MOVWF	EepromRamAddr				;EepromRamAddr = FanSpeedPresetToLoad

	MOVLW	0x01
	MOVWF	EepromAmount				;1 Byte to save
	CALL	EepromWrite					;Perform the write (Returns in Bank 1)
	CALL	FanSpeedErrorTrap			;Test for eeprom errors and trap them (Returns in Bank 1)

	MOVF	FanSpeedPresetToLoad, W
	MOVWF	FanSpeedPreset				;Restore FanSpeedPreset to the correct value after we used it for a scratch variable

	RETURN		

;************************************************************************************************************************************
; FanSpeedSavePreset: Saves the currently active FanSpeedPreset slot data from FanSpeedValues to eeprom if there were changes		*
;					  made to the slot. If ifFanSpeedChanged is clear then the data is not saved as the values in eeprom are 		*
;					  still upto date (prevents eeprom wear) - Clears the ifFanSpeedChanged flag after saving as saving will 		*
;					  make the eeprom data upto date																				*
;					  Only to be called when fFanSpeedBusy is clear ! Calling when another write is in progress will result			*
;					  in a fatal error trap!!																						*
;					  Entry -> Bank X																								*
;					  Exit  -> Bank 1																								*
;************************************************************************************************************************************
FanSpeedSavePreset

	CALL	iFanSpeedCalcEepromAddr		;Calcalate the required eeprom address, save to EepromAddr (Returns in Bank 1)

	BTFSS	ifFanSpeedChanged			;Values of this preset slot were changed since last save/load ?
	RETURN								;No? then return - Data in eeprom is upto date 

	MOVLW	FanSpeedValues
	MOVWF	EepromRamAddr				;Load the ram address of FanSpeedValues into EepromRamAddr

	MOVLW	0x06
	MOVWF	EepromAmount				;6 bytes to save
	CALL	EepromWrite					;Perform the write (Returns in Bank 1)
	CALL	FanSpeedErrorTrap			;Test for eeprom errors and trap them (Returns in Bank 1)

	BCF		ifFanSpeedChanged			;Clear the changed flag - Eeprom is upto date now

	RETURN

;************************************************************************************************************************************
; FanSpeedErrorTrap: Checks to see an error occured during an eeprom operation. The fEepromError flag is tested. If the flag is set *
;				     the eeprom fatal ErrorTrap is activated. Otherwise the function returns										*
;					 Entry -> Bank X																								*
;					 Exit  -> Bank 1																								*
;************************************************************************************************************************************
FanSpeedErrorTrap

	BANKSEL	EepromFlags					;Bank 1
	MOVLW	cErrorTrap_Eeprom			;Assume a fatal eeprom error occured
	BTFSC	fEepromError				;Did an eeprom error occur ? 
	CALL	ErrorTrapW					;Yes? then trap it
	RETURN								;No? then just return	 
	
;************************************************************************************************************************************
; iFanSpeedCalcEepromAddr: Calculates the required eeprom address to save/load the preset number in FanSpeedPreset and saves the	*
;						   result directly in EepromAddr. Only accepts preset values 1, 2 and 3. A fatal error is trapped if this	*
;						   function is called with another value in FanSpeedPreset													*
;						   Entry -> Bank X																							*
;						   Exit  -> Bank 1																							*
;************************************************************************************************************************************
iFanSpeedCalcEepromAddr

	BANKSEL	FanSpeedPreset				;Bank 1
	MOVLW	0x01
	XORWF	FanSpeedPreset, W			;FanSpeedPreset == 1 ? => Z flag set on match
	MOVLW	0x00						;Assume match
	BTFSC	STATUS, Z					;Is it a match?
	GOTO	iFanSpeedCalcSaveEepromAddr	;Yes? then store the result
	
	MOVLW	0x02
	XORWF	FanSpeedPreset, W			;FanSpeedPreset == 2 ? => Z flag set on match
	MOVLW	0x06						;Assume match
	BTFSC	STATUS, Z					;Is it a match?
	GOTO	iFanSpeedCalcSaveEepromAddr	;Yes? then store the result

	MOVLW	0x03
	XORWF	FanSpeedPreset, W			;FanSpeedPreset == 3 ? => Z flag set on match
	MOVLW	0x0C						;Assume match
	BTFSC	STATUS, Z					;Is it a match?
	GOTO	iFanSpeedCalcSaveEepromAddr	;Yes? then store the result
										;No match found? then FanSpeedPreset holds an invalid value !
	MOVLW	cErrorTrap_Preset			;Load the ErrorTrap code 
	CALL	ErrorTrapW					;And trap this fatal (and normally impossible) error

iFanSpeedCalcSaveEepromAddr

	MOVWF	EepromAddr					;Save the result in W
	RETURN	

;************************************************************************************************************************************
; FanSpeedSelectLast: Changes FanSpeedSelectedValue to select Fan 6																	*
;					  Entry -> Bank X																								*
;					  Exit  -> Bank 1																								*
;************************************************************************************************************************************
FanSpeedSelectLast

	BANKSEL	FanSpeedSelectedValue		;Bank 1
	MOVLW	0x05
	MOVWF	FanSpeedSelectedValue		;5 = Fan 6

	RETURN

;************************************************************************************************************************************
; FanSpeedSelectNext: Changes FanSpeedSelectedValue to select the next fan. Rolls back to Fan 1 after Fan 6							*
;					  Entry -> Bank X																								*
;					  Exit  -> Bank 1																								*
;************************************************************************************************************************************
FanSpeedSelectNext

	BANKSEL	FanSpeedSelectedValue		;Bank 1
	INCF	FanSpeedSelectedValue, F	;Increment FanSpeedSelectedValue. Make it select the next fan
	
	MOVLW	0x06
	XORWF	FanSpeedSelectedValue, W
	BTFSC	STATUS, Z					;FanSpeedSelectedValue == 6 ? 
	CLRF	FanSpeedSelectedValue		;Yes ? then reset FanSpeedSelectedValue to Fan 1
										;No ? then just return
	RETURN								 

;************************************************************************************************************************************
; FanSpeedGetSelectedW: Retrieves the FanSpeedValue of the fan selected with FanSpeedSelectedValue in W								*
;						Entry -> Bank X																								*
;						Exit  -> Bank 1																								*
;************************************************************************************************************************************
FanSpeedGetSelectedW

	BANKISEL FanSpeedValues				;Indirect addressing Bank 0/1

	BANKSEL	FanSpeedSelectedValue		;Bank 1
	MOVF	FanSpeedSelectedValue, W
	ADDLW	FanSpeedValues
	MOVWF	FSR							;Point indirect addressing to FanSpeedValues[FanSpeedSelectedValue]
	
	MOVF	INDF, W						;Copy the FanSpeedValue to W
	RETURN								;And return with it 

;************************************************************************************************************************************
; FanSpeedDecSelected: Decreases the FanSpeedValue of the fan selected with FanSpeedSelectedValue according to the following 		*
;					   pattern: 100 - 98 - ... - 22 - 20 - 0 , Rolls back to 100 after 0											*
;					   Sets the ifFanSpeedChanged flag - Currently loaded preset slot is changed now								*
;					   Entry -> Bank X																								*
;					   Exit  -> Bank 1																								*
;************************************************************************************************************************************
FanSpeedDecSelected

	CALL	FanSpeedGetSelectedW		;Returns the FanSpeedValue of the selected fan in W...
										;... Also points indirect adressing to the FanspeedValue we need (Returns in Bank 1)
	MOVLW	0x02
	SUBWF	INDF, F						;Subtract 2 from the FanSpeedValue
	
	MOVLW	d'100'						;Assume we went under 0 (borrow), so preload 100...
	BTFSS	STATUS, C					;Borrow ? 
	MOVWF	INDF						;Yes? then FanSpeedValue = 100
										;No? 
	MOVLW	d'18'
	XORWF	INDF, W						
	BTFSC	STATUS, Z					;FanSpeedValue == 18? 		
	CLRF	INDF						;Yes? then FanSpeedValue = 0;
										;No? then leave it alone
	BSF		ifFanSpeedChanged			;We've made changes to the currently loaded preset slot, set changed flag

	RETURN					

;************************************************************************************************************************************
; FanSpeedPresetSelectLast: Clears FanSpeedPresetToLoad, this selects "no operation"												*
;							Entry -> Bank X																							*
;							Exit  -> Bank 1																							*
;************************************************************************************************************************************
FanSpeedPresetSelectLast

	BANKSEL	FanSpeedPresetToLoad		;Bank 1
	CLRF	FanSpeedPresetToLoad		

	RETURN

;************************************************************************************************************************************
; FanSpeedPresetSelectNext: Changes FanSpeedPresetToLoad to select the next slot to load - 1, 2, 3  or  4 (save current). Rolls 	*
;							back to 0 (no operation) after 4																		*
;							Entry -> Bank X																							*
;							Exit  -> Bank 1																							*
;************************************************************************************************************************************
FanSpeedPresetSelectNext

	BANKSEL	FanSpeedPresetToLoad		;Bank 1
	INCF	FanSpeedPresetToLoad, F		;Increment FanSpeedPresetToLoad, Make it select the next slot to load

	MOVLW	0x05
	XORWF	FanSpeedPresetToLoad, W
	BTFSC	STATUS, Z					;FanSpeedPresetToLoad == 6 ? 
	CLRF	FanSpeedPresetToLoad		;Yes ? then reset FanSpeedPresetToLoad to 0
										;No ? then just return
	RETURN

;************************************************************************************************************************************

	END		
