;************************************************************************************************************************************
; Interface.asm : Code file for the Interface code																					*
;************************************************************************************************************************************
	
	#include "General.inc"		;Include project settings
	#include "FanSpeed.inc"		;Interface manipulates the fanspeeds...
	#include "Display.inc"		;For displaying the interface
	#include "PWM.inc"			;For generating a new PWM table on load/speed change
	#include "ErrorTrap.inc"	;For trapping fatal errors

;************************************************************************************************************************************
; Equ's and Defines																													*
;************************************************************************************************************************************
			
#Define 	icInterface_Display		0x00	;Interface mode display: Interface mode to display current speeds
#Define		icInterface_Preset		0x01	;Interface mode preset: Interface mode to load and save presets
#Define		icInterface_Speed		0x02	;Interface mode speed: Changes speed of the currently selected fan

#Define		icInterfaceDisplayTime	0x02	;The time, in seconds, between display mode updates
#Define 	icInterfaceSpeedTime	0x01	;The time, in seconds, between speed mode updates
#Define		icInterfaceFanStartTime	0x05	;The time, in seconds, The fans are ran @ 100% after loading a new preset

#Define 	icInterfaceSwitchPreset	PORTB,6	;The switch input used to select cInterface_Preset mode 
#Define		icInterfaceSwitchSpeed	PORTB,7	;The switch input used to select cInterface_Speed mode

;************************************************************************************************************************************
; Exports																															*
;************************************************************************************************************************************

	;Registers
	;...Add register exports here...

	;Functions
	GLOBAL	InterfaceInit
	GLOBAL	InterfaceTick

;************************************************************************************************************************************
; Variables																															*
;************************************************************************************************************************************
Bank0				UDATA					;Uninitialized variables in Bank 0
iInterfaceMode		RES 1					;The operating mode the Interface is in	for the current Tick				
iInterfaceLastMode	RES 1					;The operating mode the Interface was in the previous Tick

;************************************************************************************************************************************
PROG1	CODE	;Code 																												*
;************************************************************************************************************************************
; InterfaceInit: Initializes the interface. Sets PORTB<6:7> as inputs. 																*
;				 Then starts up the interface in Display mode...																	*
;				 iInterfaceMode is set to icInterface_Display, then iInterfaceDisplayInit and iInterfaceDisplayTick are called		*
;				 making the device startup in Display mode - selecting/displaying Fan 1												*
;				 Entry -> Bank X																									*
;				 Exit  -> Bank 1																									*
;************************************************************************************************************************************
InterfaceInit

	BANKSEL	TRISB							;Bank 1
	BSF		TRISB, 7
	BSF		TRISB, 6						;Turn PORTB<6:7> into inputs

	MOVLW	cDisplayI
	DisplaySetW	DisplayHund					;Make hundreds display display a 'I'
	MOVLW	cDisplayN
	DisplaySetW	DisplayTens					;Make tens display display a 'n'
	MOVLW	cDisplayI
	DisplaySetW	DisplayOnes					;Make ones display display a 'I'  (InI)
	CALL	iInterfaceFanStartup			;Run all fans at full speed for icInterfaceFanStartTime seconds to make sure they get started (Returns in Bank 1)...
											;...display will show "F InI" meaning (F)ull speed, (InI)tialisation
	BANKSEL	iInterfaceMode					;Bank 0
	MOVLW	icInterface_Display
	MOVWF	iInterfaceMode					;Set iInterfaceMode to icInterface_Display at startup

	CALL	iInterfaceDisplayInit			;Hard call the display mode init code (Returns in Bank 1)
	CALL	iInterfaceDisplayTick			;Hard call the display mode tick code - Now Fan 1 is selected/displayed (Returns in Bank 1)

	RETURN
	
;************************************************************************************************************************************
; InterfaceTick: The main interface function. Compares the currently user selected interface mode with the mode used in the 		*
;				 previous call. When the mode changed, the previous mode's shutdown and the new mode's initialisation are called.	*
;			     If the mode did not change, then the active mode's Tick function is called.										*
;				 Entry -> Bank X																									*
;				 Exit  -> Bank ?																									*
;************************************************************************************************************************************
InterfaceTick

	BANKSEL	iInterfaceMode					;Bank 0
	MOVF	iInterfaceMode, W
	MOVWF	iInterfaceLastMode				;Backup the Interface mode from the previous tick

	MOVLW	icInterface_Display
	MOVWF	iInterfaceMode					;Assume the current tick's interface mode is cInterface_Display

	MOVLW	icInterface_Preset				;Preload cInterface_Preset mode, in case our assumption is wrong
	BTFSC	icInterfaceSwitchPreset			;Is the switch in the Preset mode position ? 
	MOVWF	iInterfaceMode					;Yes ? then set iInterfaceMode to Preset mode

	MOVLW	icInterface_Speed				;Preload cInterface_Speed mode, in case our assumption is wrong 
	BTFSC	icInterfaceSwitchSpeed			;Is the switch in the Speed mode position ? 
	MOVWF	iInterfaceMode					;Yes ? then set iInterfaceMode to Speed mode
											;iInterfaceMode now holds the current interface mode; iInterfaceLastMode holds last tick's mode
	MOVF	iInterfaceLastMode, W
	XORWF	iInterfaceMode, W
	BTFSS	STATUS, Z						;Current tick's mode == Last tick's mode ? 
	GOTO	iInterfaceModeChange			;No? Then handle the change of Interface mode
											;Yes? Then call the active mode's tick...
	MOVLW	icInterface_Display
	XORWF	iInterfaceMode, W
	BTFSC	STATUS, Z						;iInterfaceMode == icInterface_Display ? 
	CALL	iInterfaceDisplayTick			;Yes? then call the display mode tick 
										
	MOVLW	icInterface_Preset
	XORWF	iInterfaceMode, W
	BTFSC	STATUS, Z						;iInterfaceMode == icInterface_Preset ? 
	CALL	iInterfacePresetTick			;Yes? then call the preset mode tick
										
	MOVLW	icInterface_Speed
	XORWF	iInterfaceMode, W
	BTFSC	STATUS, Z						;iInterfaceMode == icInterface_Speed ? 
	CALL	iInterfaceSpeedTick				;Yes? then call the speed mode tick

	RETURN									;Done for this tick
											
iInterfaceModeChange						;In case the interface mode changed, we need to run the shutdown code for the previous mode...
											;...and the init code for the new mode... First the shutdowns
	MOVLW	icInterface_Display
	XORWF	iInterfaceLastMode, W
	BTFSC	STATUS, Z						;iInterfaceLastMode == icInterface_Display ?
	CALL	iInterfaceDisplayEnd			;Yes? then call the display mode shutdown

	MOVLW	icInterface_Preset
	XORWF	iInterfaceLastMode, W
	BTFSC	STATUS, Z						;iInterfaceLastMode == icInterdace_Preset ? 
	CALL	iInterfacePresetEnd				;Yes? then call the preset mode shutdown

	MOVLW	icInterface_Speed
	XORWF	iInterfaceLastMode, W
	BTFSC	STATUS, Z						;iInterfaceLastMode == icInterface_Speed ? 
	CALL	iInterfaceSpeedEnd				;Yes? then call the speed mode shutdown
											;Now the init's...
	MOVLW	icInterface_Display
	XORWF	iInterfaceMode, W
	BTFSC	STATUS, Z						;iInterfaceMode == icInterface_Display ?
	CALL	iInterfaceDisplayInit			;Yes? then call the display mode init

	MOVLW	icInterface_Preset
	XORWF	iInterfaceMode, W
	BTFSC	STATUS, Z						;iInterfaceMode == icInterface_Preset ? 
	CALL	iInterfacePresetInit			;Yes? then call the preset mode init

	MOVLW	icInterface_Speed
	XORWF	iInterfaceMode, W
	BTFSC 	STATUS, Z						;iInterfaceMode == icInterface_Speed ?
	CALL	iInterfaceSpeedInit				;Yes? then call the speed mode init

	GOTO	InterfaceTick					;When the mode change is performed, immediately run another tick

;************************************************************************************************************************************
; iInterfaceDisplayInit: Selects Fan 6 for display/manipulation (so the next call to iInterfaceDisplayTick will select and display	*
;						 Fan 1) and clears the TimerSeconds counter so iInterfaceDisplayTick will update immediately.				*
;						 Entry -> Bank X																							*
;						 Exit  -> Bank 1																							*
;************************************************************************************************************************************
iInterfaceDisplayInit

	CALL	FanSpeedSelectLast				;Select Fan 6 for display/manipulation	(Returns in Bank 1)
	CLRF	TimerSeconds					;And clear TimerSeconds
	RETURN

;************************************************************************************************************************************
; iInterfaceDisplayTick: When TimerSeconds reached 0, The function displays the next fan's speed and reloads TimerSeconds.			*
;						 If TimerSeconds isn't 0 yet, the function returns without doing anything.									*
;						 Entry -> Bank X																							*
;						 Exit  -> Bank 1																							*
;************************************************************************************************************************************
iInterfaceDisplayTick

	BANKSEL	TimerSeconds					;Bank 1
	MOVF	TimerSeconds, F
	BTFSS	STATUS, Z						;TimerSeconds == 0 ?
	RETURN									;No? Then just return
											;Yes? 
	CALL	FanSpeedSelectNext				;Then select the next fan for display/manipulation (Returns in Bank 1)

	MOVF	FanSpeedSelectedValue, W
	CALL	DisplaySetFanW					;Set the fan display to the newly selected fan (Returns in Bank 1)

	CALL	FanSpeedGetSelectedW			;Retrieve the speed of the newly selected fan in W (Returns in Bank 1)
	MOVWF	DisplayValue					;And store it in the DisplayValue
	CALL	DisplayConvert					;And convert it to decimal and display it (Returns in Bank 1)

	MOVLW	icInterfaceDisplayTime			
	CALL	DisplaySetTimerSecondsW			;And reload the TimerSeconds timer (Returns in Bank 1)

	RETURN										
	
;************************************************************************************************************************************
; iInterfaceDisplayEnd:	**NOTHING FOR THE MOMENT**																					*
;						Entry -> Bank ?																								*
;						Exit  -> Bank ?																								*
;************************************************************************************************************************************
iInterfaceDisplayEnd

	;...Add any future display mode shutdown code here...

	RETURN

;************************************************************************************************************************************
; iInterfacePresetInit:	Sets FanSpeedPresetToLoad to 0 (no operation) so the next call to iInterfacePresetTick will select 			*
;						slot 1 for load and clears the TimerSeconds counter so iInterfacePresetTick will update immediately.		*
;						Entry -> Bank X																								*
;						Exit  -> Bank 1																								*
;************************************************************************************************************************************
iInterfacePresetInit

	CALL	FanSpeedPresetSelectLast		;Select no operation (0) (Returns in Bank 1)
	CLRF	TimerSeconds					;And clear TimerSeconds
	RETURN

;************************************************************************************************************************************
; iInterfacePresetTick:	When TimerSeconds reached 0, the functions selects the next preset slot for load/save current and updates 	*
;						the display. If TimerSeconds isn't 0 yet, the functions returns without doing anything						*
;						Entry -> Bank X																								*
;						Exit  -> Bank 1																								*
;************************************************************************************************************************************
iInterfacePresetTick

	BANKSEL	TimerSeconds					;Bank 1
	MOVF	TimerSeconds, F
	BTFSS	STATUS, Z						;TimerSeconds == 0 ?
	RETURN									;No? then just return
											;Yes?
	MOVLW	cErrorTrap_PresetTimeOut		;Preload the ErrorTrap code for a fFanSpeedBusy timeout - assume error
	BTFSC	fFanSpeedBusy					;Is the fanspeed busy flag set ?
	CALL	ErrorTrapW						;Yes? It has been set for too long , trap this fatal error
 											;No? 
	CALL	FanSpeedPresetSelectNext		;Select the next preset slot for load/save current (Returns in Bank 1)
		
	MOVF	FanSpeedPresetToLoad, F
	BTFSS	STATUS, Z						;FanSpeedPresetToLoad == 0 ? (No operation)
	GOTO	iInterfacePresetLoadSave		;No? then we eighter need to display loading or saving
											;Yes? then we need to display "No Operation" (0)
	MOVLW	0x00
	DisplaySetW		DisplayFan				;Make the fan display display a 0
	MOVLW	cDisplayOff
	DisplaySetW		DisplayHund
	MOVLW	cDisplayOff
	DisplaySetW		DisplayTens
	MOVLW	cDisplayOff
	DisplaySetW		DisplayOnes				;Turn off all the other displays 

	GOTO	iInterfacePresetTickEnd			;Done for this tick
	
iInterfacePresetLoadSave					;We eighter need to display loading or saving ...

	MOVLW	0x04
	XORWF	FanSpeedPresetToLoad, W
	BTFSS	STATUS, Z						;FanSpeedPresetToLoad == 4 (save) ?	
	GOTO	iInterfacePresetLoad			;No? then its loading
											;Yes? saving...we need to display 'S <current slotnumber>'
	MOVLW	cDisplayS
	DisplaySetW		DisplayFan				;Make the fan display display a S

	MOVF	FanSpeedPreset, W
	MOVWF	DisplayValue					;Copy the current preset slot number to the DisplayValue
	CALL	DisplayConvert					;And convert it to decimal and display it (Returns in Bank 1)

	GOTO	iInterfacePresetTickEnd			;Done for this tick

iInterfacePresetLoad						;Loading...we need to display 'L <Selected slotnumber>'

	MOVLW	cDisplayL
	DisplaySetW		DisplayFan				;Make the fan display display a L

	MOVF	FanSpeedPresetToLoad, W
	MOVWF	DisplayValue					;Copy the currently selected preset slotnumber to load to the DisplayValue
	CALL	DisplayConvert					;And convert it to decimal and display it (Returns in Bank 1)

iInterfacePresetTickEnd

	MOVLW	icInterfaceDisplayTime			
	CALL	DisplaySetTimerSecondsW			;Reload the TimerSeconds timer (Returns in Bank 1)
	
	RETURN

;************************************************************************************************************************************
; iInterfacePresetEnd: Performs the selected action in FanSpeedPresetToLoad															*
;					   0: No operation, does nothing																				*
;					   1,2 or 3: Load the relevant slot number																		*
;					   4: Save the currently loaded slot 																			*	
;					   Entry -> Bank X																								*
;					   Exit  -> Bank 1																								*
;************************************************************************************************************************************
iInterfacePresetEnd

	BANKSEL	FanSpeedPresetToLoad			;Bank 1
	MOVF	FanSpeedPresetToLoad, F
	BTFSC	STATUS, Z						;FanSpeedPresetToLoad == 0 ?
	RETURN									;Yes? then return without taking action

	MOVLW	0x04
	XORWF	FanSpeedPresetToLoad, W
	BTFSS	STATUS, Z						;FanSpeedPresetToLoad == 4 ?
	GOTO	iInterfacePerformLoad			;No? then perform the load
											;Yes? 
	CALL	FanSpeedSavePreset				;Then perform the save (Returns in Bank 1)	
	RETURN									;And return

iInterfacePerformLoad

	CALL	FanSpeedLoadPreset				;Perform the Load (Returns in Bank 1)
	CALL	PWMGenerateTable				;Generate a new PWM table to immediately use the new speeds (Returns in Bank 0)

	BANKSEL	DisplayHund						;Bank 1
	MOVLW	cDisplayL
	DisplaySetW	DisplayHund					;Make hundreds display display a 'L'
	MOVLW	cDisplayOff
	DisplaySetW	DisplayTens					;Turn off the tens display, ones display still display the preset slot number we have loaded
	CALL	iInterfaceFanStartup			;Run all fans at full speed for icInterfaceFanStartTime seconds to make sure they get started (Returns in Bank 1)...
											;...display will show "F L X" meaning (F)ull speed, (L)oading X = preset slot number
	RETURN

;************************************************************************************************************************************
; iInterfaceSpeedInit: Reloads TimerSeconds to make sure it will take icInterfaceSpeedTime seconds before the first speed change	*
;					   Entry -> Bank X																								*
;					   Exit  -> Bank 1																								*
;************************************************************************************************************************************
iInterfaceSpeedInit 

	MOVLW	icInterfaceSpeedTime
	CALL	DisplaySetTimerSecondsW			;Reload TimerSeconds timer (Returns in Bank 1)

	RETURN

;************************************************************************************************************************************
; iInterfaceSpeedTick: When TimerSeconds reached 0, the function will increase the selected fan's speed by 2%, update the display,	*
;					   and reload TimerSeconds. If TimerSeconds isn't 0 yet, the function returns without doing anything			*
;					   Entry -> Bank X																								*
;					   Exit  -> Bank ?																								*
;************************************************************************************************************************************
iInterfaceSpeedTick

	BANKSEL	TimerSeconds					;Bank 1
	MOVF	TimerSeconds, F
	BTFSS	STATUS, Z						;TimerSeconds == 0 ?
	RETURN									;No? Then just return
											;Yes? 
	CALL	FanSpeedDecSelected				;Decrease the speed of the selected fan by 2% (handles 0->100 and 20->0 automatically) (Returns in Bank 1)
	CALL	PWMGenerateTable				;Generate a new PWM table - The next PWMCycle call will immediately produce the new speed (Returns in Bank 0)

	CALL	FanSpeedGetSelectedW			;Retrieve the new speed of the selected fan in W (Returns in Bank 1)
	MOVWF	DisplayValue					;And store it in the DisplayValue
	CALL	DisplayConvert					;And convert it to decimal and display it (Returns in Bank 1)

	MOVLW	icInterfaceSpeedTime
	CALL	DisplaySetTimerSecondsW			;And reload the TimerSeconds timer (Returns in Bank 1)

	RETURN

;************************************************************************************************************************************																											
; iInterfaceSpeedEnd: **NOTHING FOR THE MOMENT**																					*
;					  Entry -> Bank ?																								*
;					  Exit  -> Bank ?																								*
;************************************************************************************************************************************
iInterfaceSpeedEnd

	;...Add any future speed mode shutdown code here...

	RETURN

;************************************************************************************************************************************
; iInterfaceFanStartup: Displays a F (for Full speed) on the FanDisplay and runs all fans at 100% speed during 						*
;						icInterfaceFanStartTime	seconds. This is to make sure the fans get started before they are regulated down 	*
;						to their speed setting.																						*
;						The interface is locked during this time.																	*
;						Entry -> Bank X																								*
;						Exit  -> Bank 1																								*
;************************************************************************************************************************************
iInterfaceFanStartup

	BANKSEL	PORTB							;Bank 0
	MOVLW	0xFF
	MOVWF	PORTB							;Set all PORTB outputs high => fans at full speed

	MOVLW	icInterfaceFanStartTime
	CALL	DisplaySetTimerSecondsW			;Load TimerSeconds timer (Returns in Bank 1)

	MOVLW	0x0F
	DisplaySetW	DisplayFan					;Make the Fan display display an 'F'

	MOVF	TimerSeconds, F
	BTFSS	STATUS, Z						;TimerSeconds == 0?
	GOTO	$-2								;No? Then test again

	RETURN									;Yes? then return

;************************************************************************************************************************************

	END		
