;************************************************************************************************************************************
; PWM.asm : Code file for the software PWM code																						*
; 			FanSpeedValues <=> IO MAP:	FanSpeedValues[0] = PORTB, 0																*
;										...																							*
;										FanSpeedValues[5] = PORTB, 5																*
;************************************************************************************************************************************
	
	#include "General.inc"		;Include project settings
	#include "FanSpeed.inc"		;For access to the FanSpeedValues
	#include "Display.inc"		;For access to the timer registers

;************************************************************************************************************************************
; Equ's and Defines																													*
;************************************************************************************************************************************

#Define		icPWMTableSize	d'41'			;Size, in bytes, of the PWMTable array

;************************************************************************************************************************************
; Exports																															*
;************************************************************************************************************************************

	;Registers
	;...Add register exports here...

	;Functions
	GLOBAL	PWMInit
	GLOBAL	PWMGenerateTable
	GLOBAL	PWMCycle

;************************************************************************************************************************************
; Variables																															*
;************************************************************************************************************************************
Bank0					UDATA				;Uninitialized variables in Bank 0
iPWMTable				RES icPWMTableSize	;The software PWM data array
iPWMTmp					RES 1				;Scratch variable

;************************************************************************************************************************************
; Macros																															*
;************************************************************************************************************************************
; iPWMSpeedValue:	Performs:  BSF PWMTable[(FanSpeedValues[FanNr] - 20) / 2], FanNr												*
;							   Does nothing when FanSpeedValues[FanNr] == 0															*
;							   Entry -> Bank X																						*
;							   Exit  -> Bank 0																						*
;************************************************************************************************************************************
iPWMSpeedValue	macro	FanNr
	LOCAL	iPWMSpeedValueEnd				;Define macro labels as local 

	BANKSEL	FanSpeedValues					;Bank 1
	MOVF	FanSpeedValues + FanNr, W		;W = FanSpeedValues[FanNr] => affects Z flag*
	
	BANKSEL	iPWMTmp							;Bank 0
	MOVWF	iPWMTmp							;iPWMTmp = FanSpeedValues[FanNr]

	BTFSC	STATUS, Z						;W == 0 ? *
	GOTO	iPWMSpeedValueEnd				;Yes? then end

 	MOVLW	d'20'
	SUBWF	iPWMTmp, F						;iPWMTmp = FanSpeedValues[FanNr] - 20
	
	BCF		STATUS, C						;Clear carry for RRF instruction
	RRF		iPWMTmp, W						;W = (FanSpeedValues[FanNr] - 20) / 2

	BANKISEL iPWMTable						;Indirect addressing bank 0/1
	ADDLW	iPWMTable
	MOVWF	FSR								;Point indirect addressing to PWMTable[(FanSpeedValues[FanNr] - 20) / 2]

	BSF		INDF, FanNr						;BSF PWMTable[(FanSpeedValues[FanNr] - 20) / 2], FanNr
	
iPWMSpeedValueEnd

	Endm

;************************************************************************************************************************************
PROG1	CODE	;Code 																												*
;************************************************************************************************************************************
; PWMInit: Sets PORTB<0:5> as outputs and calls PWMGenerateTable.																	*
;		   ***PWMGenerate table requires FanSpeed data to be loaded, This function needs to be called AFTER FanSpeedInit***			*
;		   Entry -> Bank X																											*
;		   Exit  -> Bank 0																											*
;************************************************************************************************************************************
PWMInit

	BANKSEL	PORTB							;Bank 0
	CLRF	PORTB							;Make sure PORTB is cleared

	BANKSEL	TRISB							;Bank 1
	MOVLW	b'11000000'
	ANDWF	PORTB, F						;Turn PORTB<0:5> into outputs, without affecting PORTB<6:7>

	CALL	PWMGenerateTable				;And generate the PWM table (Returns in Bank 0)

	RETURN

;************************************************************************************************************************************
; PWMGenerateTable: Generates the data required in the iPWMTable out of the FanSpeedValues											*
;					Entry -> Bank X																									*
;					Exit  -> Bank 0 																								*
;************************************************************************************************************************************
PWMGenerateTable
											;First we need to clear the entire table
	BANKSEL	iPWMTable						;Bank 0							
	MOVLW	icPWMTableSize
	MOVWF	iPWMTmp							;Loop icPWMTableSize times

	BANKISEL iPWMTable						;Indirect addressing Bank 0/1	
	MOVLW	iPWMTable
	MOVWF	FSR								;Point indirect addressing to PWMTable[0]

iPWMClearTableNext

	CLRF	INDF							;Clear current byte
	INCF	FSR, F							;Increment indirect addressing to next table byte
	
	DECFSZ	iPWMTmp, F						;Are we done?
	GOTO	iPWMClearTableNext				;No? then clear the next byte
										 	;Yes? then continue
	iPWMSpeedValue	0x00					;BSF PWMTable[(FanSpeedValues[0] - 20) / 2], 0
	iPWMSpeedValue  0x01					;BSF PWMTable[(FanSpeedValues[1] - 20) / 2], 1
	iPWMSpeedValue 	0x02					;BSF PWMTable[(FanSpeedValues[2] - 20) / 2], 2
	iPWMSpeedValue  0x03					;BSF PWMTable[(FanSpeedValues[3] - 20) / 2], 3
	iPWMSpeedValue  0x04					;BSF PWMTable[(FanSpeedValues[4] - 20) / 2], 4
	iPWMSpeedValue  0x05					;BSF PWMTable[(FanSpeedValues[5] - 20) / 2], 5 (Bank 0)
											;Now OR the values from PWMTable[98%] down to PWMTable[0%]
	MOVLW	icPWMTableSize - 1
	MOVWF	iPWMTmp							;Loop icPWMTableSize - 1 times (98% value itself does not get OR'ed)
											;Indirect addressing Bank already set to 0/1
	MOVLW	iPWMTable + (icPWMTableSize - 1)	
	MOVWF	FSR								;Point indirect addressing to PWMTable[98%]	

iPWMTableOrNext

	MOVF	INDF, W							;W = PWMTable[X]							
	DECF	FSR, F							
	IORWF	INDF, F							;PWMTable[X - 1] = PWMTable[X - 1] OR PWMTable[X]

	DECFSZ	iPWMTmp, F						;Are we done?
	GOTO	iPWMTableOrNext					;No? then OR the next pair				
	RETURN									;Yes? then return
	
;************************************************************************************************************************************
; PWMCycle:	Generates the PWM signal for all 6 fans during approximately 0.1 second, then returns.									*
;			Entry -> Bank X																											*
;			Exit  -> Bank 0																											*
;************************************************************************************************************************************
PWMCycle

	BANKSEL	TimerInt						;Bank 1
	MOVLW	d'26'
	MOVWF	TimerInt						;Set TimerInt to start counting down from '26' (25* 4mS =>  100mS)

	BANKSEL	PORTB							;Bank 0
	BANKISEL iPWMTable						;Indirect addressing Bank 0/1

	MOVF	iPWMTable, W
	MOVWF	PORTB							;PORTB = PWMTable[0%] 										   				Tcopy(0%)

	GOTO	$+1								;Takes 2 cycles, required for accurate timing								+2

iPWMCycleRestart

	MOVLW	icPWMTableSize - 1																							+3																				
	MOVWF	iPWMTmp							;Loop icPWMTableSize -1 times - we will copy the entire PWMTable to PORTB...+4
											;...except for PWMTable[0%] wich is already on PORTB 
	MOVLW	iPWMTable + 1																								+5
	MOVWF	FSR								;Point indirect addressing to PWMTable[20%]									+6

	BANKSEL	TimerInt						;Bank 1																		+8
	MOVF	TimerInt, F						;TimerInt == 0? => Affects Z flag *											+9
	
	BANKSEL	PORTB							;Bank 0																		+11
	BTFSC	STATUS, Z						;TimerInt == 0? *															+13
	GOTO	iPWMCycleEnd					;Yes? Then end 	
	
	NOP																													+14
	GOTO	$+1																											+16
	GOTO	$+1																											+18
	GOTO	$+1																											+20	
	GOTO	$+1																											+22
	GOTO	$+1																											+24
	GOTO	$+1																											+26
	GOTO	$+1																											+28	
	GOTO	$+1																											+30	
	GOTO	$+1																											+32	
	GOTO	$+1																											+34
	GOTO	$+1																											+36
	GOTO	$+1																											+38
	GOTO	$+1																											+40
	GOTO	$+1																											+42
	GOTO	$+1																											+44
	GOTO	$+1																											+46
	GOTO	$+1																											+48
	GOTO	$+1																											+50
	GOTO	$+1																											+52
	GOTO	$+1																											+54
	GOTO	$+1																											+56
	GOTO	$+1																											+58

iPWMCycleNext

	MOVF	INDF, W							;																			+5			  +59	
	MOVWF	PORTB							;PORTB = PWMTable[X%]														+6 TCopy(X%)  +60		
	INCF	FSR, F							;Increment indirect addressing for next byte								+1	

	DECFSZ	iPWMTmp, F						;Copied the entire PWMTable to PORTB ? 										+2 +3
	GOTO	iPWMCycleNext					;No? then copy the next byte												+4	
											;Yes?
	NOP										;Required for accurate timing												   +4
	MOVF	iPWMTable, W					;																			   +5
	MOVWF	PORTB 							;PORTB = PWMTable[0%] 							 						       +6 TCopy(0%)

	GOTO	iPWMCycleRestart				;																			   +2	

iPWMCycleEnd

	CLRF	PORTB							;Turn all outputs off while we are not performing PWM
	RETURN									;And return																					

;************************************************************************************************************************************
 
	END		
